@@ -57,6 +57,9 @@ class MySQL:
         self._commit()
         self._close()
 
+    # ------------------------------------------------------------------------------
+    #                                GETTER METHODS                                |
+    # ------------------------------------------------------------------------------
     @property
     def tables(self):
         """Retrieve a list of tables in the connected database"""
@@ -68,6 +71,27 @@ class MySQL:
         """Retrieve a list of databases that are accessible under the current connection"""
         return self._fetch('show databases')
 
+    def get_primary_key(self, table):
+        """Retrieve the column which is the primary key for a table."""
+        for column in self.get_schema(table):
+            if 'pri' in column[3].lower():
+                return column[0]
+
+    def get_primary_key_values(self, table):
+        """Retrieve a list of primary key values in a table"""
+        return self.select(table, self.get_primary_key(table), _print=False)
+
+    def count_rows(self, table):
+        """Get the number of rows in a particular table"""
+        return self.select(table, 'COUNT(*)', False)
+
+    def count_rows_all(self):
+        """Get the number of rows for every table in the database."""
+        return {table: self.count_rows(table) for table in self.tables}
+    # ------------------------------------------------------------------------------
+    #                                END GETTER METHODS                            |
+    # ------------------------------------------------------------------------------
+
     # ------------------------------------------------------------------------------
     # |                            HELPER METHODS                                  |
     # ------------------------------------------------------------------------------
@@ -219,6 +243,17 @@ class MySQL:
         self.execute(statement)
         self._printer('\tMySQL table ' + str(table) + ' successfully truncated')
 
+    def truncate_database(self):
+        """Drop all tables in a database."""
+        # Get list of tables
+        tables = self.tables if isinstance(self.tables, list) else [self.tables]
+        if len(tables) > 0:
+            # Join list of tables into comma separated string
+            tables_str = ', '.join([wrap(table) for table in tables])
+            self.execute('DROP TABLE ' + tables_str)
+            self._printer(len(tables), 'tables truncated')
+        return tables
+
     def drop_table(self, table):
         """Drop a table from a database."""
         self.execute('DROP TABLE ' + wrap(table))
@@ -283,15 +318,6 @@ class MySQL:
         for row in values:
             self.update(table, columns, row, (where_col, row[where_index]))
 
-    def truncate_database(self):
-        """Drop all tables in a database."""
-        # Get list of tables
-        tables = self.tables if isinstance(self.tables, list) else [self.tables]
-        tables_str = ', '.join([wrap(table) for table in tables])
-
-        self.execute('DROP TABLE ' + tables_str)
-        return tables
-
     # def create_table(self, table, data, headers=None):
     #     """Generate and execute a create table query by parsing a 2D dataset"""
     #     # TODO: Fix
@@ -342,7 +368,9 @@ class MySQL:
         # all SQL commands (split on ';')
         # remove dbo. prefixes from table names
         sql_commands = [com.replace("dbo.", '') for com in sql_file.split(';')]
-        self._printer(len(sql_commands), 'Total commands')
+        self._printer(len(sql_commands), 'Total commands (sql_file.split(";"))')
+        sql_commands = [com.replace("dbo.", '') for com in split_sql_commands(sql_file)]
+        self._printer(len(sql_commands), 'Total commands (split_sql_commands)')
 
         # Save failed commands to list
         fails = []
@@ -369,47 +397,109 @@ class MySQL:
             self._printer(len(fails), 'total failed commands')
 
             # Create a directory to save fail SQL scripts
-            fails_folder = 'fails ' + datetime.fromtimestamp(time()).strftime('%Y-%m-%d %H-%M-%S')
-            fails_dir = os.path.join(os.path.dirname(sql_script), fails_folder)
+            fails_dir = os.path.join(os.path.dirname(sql_script), 'fails')
+            if not os.path.exists(fails_dir):
+                os.mkdir(fails_dir)
+
+            fails_dir = os.path.join(fails_dir, datetime.fromtimestamp(time()).strftime('%Y-%m-%d %H-%M-%S'))
             if not os.path.exists(fails_dir):
                 os.mkdir(fails_dir)
 
             # Dump failed commands to text file in the same directory as the script
-            for count, fail in enumerate(fails):
-                fails_fname = str(os.path.basename(sql_script).strip('_fails').rsplit('.')[0]) + str(count) + '.sql'
+            for count, fail in tqdm(enumerate(fails), total=len(fails), desc='Dumping failed SQL commands to text'):
+                fails_fname = str(os.path.basename(sql_script).rsplit('.')[0]) + str(count) + '.sql'
                 txt_file = os.path.join(fails_dir, fails_fname)
 
                 # Dump to text file
-                self._printer('Fail commands dumped to', txt_file)
                 with open(txt_file, 'w') as txt:
                     txt.writelines(fail)
     # ------------------------------------------------------------------------------
     #                             END STANDALONE METHODS                           |
     # ------------------------------------------------------------------------------
 
-    # ------------------------------------------------------------------------------
-    #                                GETTER METHODS                                |
-    # ------------------------------------------------------------------------------
-    def get_primary_key(self, table):
-        """Retrieve the column which is the primary key for a table."""
-        for column in self.get_schema(table):
-            if 'pri' in column[3].lower():
-                return column[0]
 
-    def get_primary_key_values(self, table):
-        """Retrieve a list of primary key values in a table"""
-        return self.select(table, self.get_primary_key(table), _print=False)
+def split_sql_commands(text):
+    results = []
+    current = ''
+    state = None
+    for c in tqdm(text, total=len(text), desc='Parsing SQL script file', unit='chars'):
+        if state is None:  # default state, outside of special entity
+            current += c
+            if c in '"\'':
+                # quoted string
+                state = c
+            elif c == '-':
+                # probably "--" comment
+                state = '-'
+            elif c == '/':
+                # probably '/*' comment
+                state = '/'
+            elif c == ';':
+                # remove it from the statement
+                current = current[:-1].strip()
+                # and save current stmt unless empty
+                if current:
+                    results.append(current)
+                current = ''
+        elif state == '-':
+            if c != '-':
+                # not a comment
+                state = None
+                current += c
+                continue
+            # remove first minus
+            current = current[:-1]
+            # comment until end of line
+            state = '--'
+        elif state == '--':
+            if c == '\n':
+                # end of comment
+                # and we do include this newline
+                current += c
+                state = None
+            # else just ignore
+        elif state == '/':
+            if c != '*':
+                state = None
+                current += c
+                continue
+            # remove starting slash
+            current = current[:-1]
+            # multiline comment
+            state = '/*'
+        elif state == '/*':
+            if c == '*':
+                # probably end of comment
+                state = '/**'
+        elif state == '/**':
+            if c == '/':
+                state = None
+            else:
+                # not an end
+                state = '/*'
+        elif state[0] in '"\'':
+            current += c
+            if state.endswith('\\'):
+                # prev was backslash, don't check for ender
+                # just revert to regular state
+                state = state[0]
+                continue
+            elif c == '\\':
+                # don't check next char
+                state += '\\'
+                continue
+            elif c == state[0]:
+                # end of quoted string
+                state = None
+        else:
+            raise Exception('Illegal state %s' % state)
 
-    def count_rows(self, table):
-        """Get the number of rows in a particular table"""
-        return self.select(table, 'COUNT(*)', False)
+    if current:
+        current = current.rstrip(';').strip()
+        if current:
+            results.append(current)
 
-    def count_rows_all(self):
-        """Get the number of rows for every table in the database."""
-        return {table: self.count_rows(table) for table in self.tables}
-    # ------------------------------------------------------------------------------
-    #                                END GETTER METHODS                            |
-    # ------------------------------------------------------------------------------
+    return results
 
 
 # TODO: Remove prior to 1.4 release
